package schedframe.resources.computing.profiles.energy.power.ui;

import java.util.List;

import org.joda.time.DateTimeUtils;

import schedframe.Parameters;
import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.profiles.energy.ResourceEvent;
import schedframe.resources.computing.profiles.energy.ResourceEventType;
import schedframe.resources.computing.profiles.energy.power.PowerProfile;
import schedframe.resources.computing.profiles.energy.power.PowerState;
import schedframe.resources.computing.profiles.energy.power.PowerStateName;
import schedframe.resources.computing.profiles.energy.power.PowerUsage;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;
import schedframe.resources.computing.profiles.energy.power.Transition;
import simulator.DataCenterWorkloadSimulator;

public class ComputingResourcePowerInterface implements PowerInterface{

	protected PowerStateName currentPowerState;
	protected PowerProfile powerProfile;
	protected ComputingResource resource;
	
	public ComputingResourcePowerInterface(ComputingResource resource, PowerProfile powerProfile){
		this.resource = resource;
		this.powerProfile = powerProfile;
		this.currentPowerState = StandardPowerStateName.ON;
	}
	
	public boolean setPowerState(PowerStateName newState) {
		if(!supportPowerState(newState))
			return false;
			
		if(newState != currentPowerState){

			Transition t = getTransition(currentPowerState, newState);
			if(t == null){
				currentPowerState = newState;
			} else{
				if(t.getTime() <= 0 ){
					currentPowerState = newState;
				} else {
					currentPowerState = t.getName();
					DataCenterWorkloadSimulator.getEventManager().sendToResource(resource.getFullName(), t.getTime(), new ResourceEvent(ResourceEventType.UPDATE_POWER_STATE, newState, resource.getFullName()));
				}
			}

			for(ComputingResource child:resource.getChildren()){
				if(child.getPowerInterface() != null){
					child.getPowerInterface().setPowerState(currentPowerState);
				}
			}

			if(currentPowerState == StandardPowerStateName.OFF || currentPowerState == StandardPowerStateName.POWEROFF || currentPowerState == StandardPowerStateName.BOOT){
				resource.setStatus(ResourceStatus.UNAVAILABLE);
			}
			else if(currentPowerState == StandardPowerStateName.ON){
				resource.setStatus(ResourceStatus.FREE);
			}
			//notifications from all levels
			resource.handleEvent(new ResourceEvent(ResourceEventType.POWER_STATE_CHANGED, resource.getFullName()));

			//notification from last level
			//if (resource.getChildren() == null) resource.handleEvent(new EnergyEvent(EnergyEventType.POWER_STATE_CHANGED, resource));
			
			//ResourceController.traceResource(new DateTime().getMillis(), resource.getFullName(), "POWER_STATE_CHANGED", state.getName());
		}
		return true;
	}
	

	public PowerStateName getPowerState() {
		return currentPowerState;
	}

	public List<PowerState> getSupportedPowerStates(){
		return powerProfile.getSupportedPowerStates();
	}

	public boolean supportPowerState(PowerStateName state) {
		for(PowerState powerState: powerProfile.getSupportedPowerStates()){
			if(powerState.getName().getLabel().equals(state.getLabel())){
				return true;
			}
		}
		return false;
	}
	
	public double getPowerConsumption(PowerStateName state) throws NoSuchFieldException {
		double powerConsumption = 0;
		if(supportPowerState(state)){
			for(PowerState powerState: powerProfile.getSupportedPowerStates()){
				if(powerState.getName().getLabel().equals(state.getLabel())){
					powerConsumption = powerState.getPowerUsage();
					break;
				}
			}
		} else {
			throw new NoSuchFieldException("Power state not supported");
		}
		return powerConsumption;
	}

	public PowerUsage getRecentPowerUsage() {
		PowerUsage powerUsage = null;
		int lastIdx = getPowerUsageHistory().size() - 1;
		if(lastIdx >= 0)
			powerUsage = getPowerUsageHistory().get(lastIdx);
		else {	
			try {
				powerUsage = new PowerUsage(DateTimeUtils.currentTimeMillis(), getPowerConsumption(currentPowerState));
			} catch (NoSuchFieldException e) {
			}
		}
		return powerUsage;
	}
	
	public List<PowerUsage> getPowerUsageHistory(){
		return powerProfile.getPowerUsageHistory();
	}
	
	public Parameters getParameters() {
		return powerProfile.getParameters();
	}

	protected Transition getTransition(PowerStateName from, PowerStateName to){
		List<PowerState> availablePowerStates = getSupportedPowerStates();
		Transition transition = null;
		for(PowerState ps : availablePowerStates){
			if(ps.getName().getLabel().equals(from.getLabel())){
				List<Transition> transitions = ps.getTransitions();
				for(Transition t: transitions){
					if(t.getTo().getLabel().equals(to.getLabel())){
						transition = t;
						break;
					}
				}
			}
		}
		return transition;
	}
}
